<?php
//
// Check via WinRM
// Copyright (c) 2010-2022 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__) . '/../configwizardhelper.inc.php');

viawinrm_configwizard_init();

function viawinrm_configwizard_init()
{
    $name = "viawinrm";
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "1.0.2",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _("Execute plugins on a remote Windows system via WinRM"),
        CONFIGWIZARD_DISPLAYTITLE => _("Check via WinRM"),
        CONFIGWIZARD_FUNCTION => "viawinrm_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "win2022.png",
        CONFIGWIZARD_FILTER_GROUPS => array('windows'),
        CONFIGWIZARD_REQUIRES_VERSION => 500
    );
    register_configwizard($name, $args);
}


/**
 * @param string $mode
 * @param null   $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */
function viawinrm_configwizard_func($mode = "", $inargs = null, &$outargs, &$result)
{
    $wizard_name = "viawinrm";

    // Initialize return code and output
    $result = 0;
    $output = "";

    // Initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {
        case CONFIGWIZARD_MODE_GETSTAGE1HTML:

            $address = grab_array_var($inargs, "ip_address", "");
            $winrmuser = grab_array_var($inargs, "winrmuser", "");
            $winrmpassword = grab_array_var($inargs, "winrmpassword", "");

            $output = '
<h5 class="ul">' . _('Server Information') . '</h5>
<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td class="vt">
            <label>' . _('IP Address') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="ip_address" id="ip_address" value="' . encode_form_val($address) . '" class="form-control">
            <div class="subtext">' . _('The IP address or FQDNS name of the server you\'d like to monitor.') . '</div>
        </td>
      </tr>
      <tr>
        <td class="vt">
            <label>' . _('Username') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="winrmuser" id="winrmuser" value="' . encode_form_val($winrmuser) . '" class="form-control">
            <div class="subtext">' . _('The username you want to use to connect to the Windows system.') . '</div>
        </td>
      </tr>
      <tr>
        <td class="vt">
            <label>' . _('Password') . ':</label>
        </td>
        <td>
            <input type="password" size="60" name="winrmpassword" id="winrmpassword" value="' . encode_form_val($winrmpassword) . '" class="form-control">
            <div class="subtext">' . _('The password used to connect to the Windows system.') . '</div>
        </td>
    </tr>
</table>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address", "");
            $address = nagiosccm_replace_user_macros($address);
            $winrmuser = grab_array_var($inargs, "winrmuser", "");
            $winrmpassword = grab_array_var($inargs, "winrmpassword", "");

            // Check for errors
            $errors = 0;
            $errmsg = array();
            if (have_value($address) == false)
                $errmsg[$errors++] = _("No address specified.");

            if (have_value($winrmuser) == false)
              $errmsg[$errors++] = _("No username specified.");

            if (have_value($winrmpassword) == false)
              $errmsg[$errors++] = _("No password specified.");

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address");
            $winrmuser = grab_array_var($inargs, "winrmuser", "");
            $winrmpassword = grab_array_var($inargs, "winrmpassword", "");

            $ha = @gethostbyaddr($address);
            if ($ha == "")
                $ha = $address;
            $hostname = grab_array_var($inargs, "hostname", $ha);

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial", "");
            if ($services_serial != "")
                $services = json_decode(base64_decode($services_serial), true);
            if (!is_array($services)) {
                $services_default = array(
                    "ping" => 0,
                    "commands" => array(),
                );
                $services_default["commands"][0] = "on";
                $services_default["commands"][1] = "on";
                $services_default["commands"][2] = "on";
                $services = grab_array_var($inargs, "services", $services_default);
            }
            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");
            if ($serviceargs_serial != "")
                $serviceargs = json_decode(base64_decode($serviceargs_serial), true);
            if (!is_array($serviceargs)) {
                $serviceargs_default = array(
                    "commands" => array(),
                );
                for ($x = 0; $x < 6; $x++) {
                    if ($x == 0) {
                        $serviceargs_default['commands'][$x]['command'] = 'C:\\Plugins\\check_cpu.ps1';
                        $serviceargs_default['commands'][$x]['args'] = '-warning 80 -critical 90';
                        $serviceargs_default['commands'][$x]['name'] = 'Check CPU Usage';
                    } else if ($x == 1) {
                        $serviceargs_default['commands'][$x]['command'] = 'C:\\Plugins\\check_memory.ps1';
                        $serviceargs_default['commands'][$x]['args'] = '-outputType GB -metric Used -warning 4 -critical 6';
                        $serviceargs_default['commands'][$x]['name'] = 'Memory Used';
                    } else if ($x == 2) {
                        $serviceargs_default['commands'][$x]['command'] = 'C:\\Plugins\\check_volume.ps1';
                        $serviceargs_default['commands'][$x]['args'] = '-volumename C:\\ -outputType PCT -metric Used -warning 80 -critical 90';
                        $serviceargs_default['commands'][$x]['name'] = 'C Volume Used Percent';
                    } else if ($x == 3) {
                        $serviceargs_default['commands'][$x]['command'] = 'C:\\Plugins\\check_service.ps1';
                        $serviceargs_default['commands'][$x]['args'] = '-expectedstate Stopped -servicename Spooler';
                        $serviceargs_default['commands'][$x]['name'] = 'Print Spooler Service Status';
                    } else if ($x == 4) {
                        $serviceargs_default['commands'][$x]['command'] = 'C:\\Plugins\\check_process.ps1';
                        $serviceargs_default['commands'][$x]['args'] = '-processname iexplore -metric Count -warning 2 -critical 4';
                        $serviceargs_default['commands'][$x]['name'] = 'Num of iexplore processes';
                    } else {
                        $serviceargs_default['commands'][$x]['command'] = '';
                        $serviceargs_default['commands'][$x]['args'] = '';
                        $serviceargs_default['commands'][$x]['name'] = '';
                        $services['commands'][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    }
                }

                $serviceargs = grab_array_var($inargs, "serviceargs", $serviceargs_default);
            }

            $output = '
<input type="hidden" name="ip_address" value="' . encode_form_val($address) . '">
<input type="hidden" name="winrmuser" value="' . encode_form_val($winrmuser) . '">
<input type="hidden" name="winrmpassword" value="' . encode_form_val($winrmpassword) . '">
<input type="hidden" name="winrmsecurity" value="basic-http">

<h5 class="ul">' . _('Server Details') . '</h5>
<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td>
            <label>' . _('IP Address') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="ip_address" id="ip_address" value="' . encode_form_val($address) . '" class="form-control" disabled>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <label>' . _('Operating System') . ':</label>
        </td>
            <td>
                <img src="' . nagioscore_get_ui_url() . "images/logos/" . viawinrm_configwizard_get_distro_icon() . '">
                <div class="subtext">Windows</div>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <label>' . _('Host Name') . ':</label>
        </td>
        <td>
            <input type="text" size="20" name="hostname" id="hostname" value="' . encode_form_val($hostname) . '" class="form-control">
            <div class="subtext">' . _('The name you\'d like to have associated with this server.') . '</div>
        </td>
    </tr>
</table>

<h5 class="ul">' . _('Server Metrics') . '</h5>
<p>' . _('Specify which services you\'d like to monitor for the server.') . '</p>
<table class="table table-no-border table-auto-width">
    <tr>
        <td>
            <input type="checkbox" id="p" class="checkbox" name="services[ping]"  ' . is_checked(checkbox_binary($services["ping"]), "1") . '>
        </td>
        <td>
            <label class="normal" for="p">
                <b>'._('Ping').'</b><br>
                ' . _('Monitors the server with an ICMP ping.  Useful for watching network latency and general uptime.') . '
            </label>
        </td>
    </tr>
</table>

<h5 class="ul">'._('WinRM plugins').'</h5>
<p>' . _('Specify any remote plugins that should be executed/monitored on the server using WinRM. When specifying the remote plugins, make sure to put in the full path to the plugin. E.g. C:\Plugins\check_cpu.ps1') . ' </p>
<table class="adddeleterow table table-condensed table-no-border table-auto-width" style="margin-bottom: 10px;">
    <tr>
        <th></th>
        <th>' . _('Service Name') . '</th>
        <th>' . _('Remote Plugin') . '</th>
        <th>' . _('Plugin Arguments') . '</th>
        </th>
    </tr>';

            for ($x = 0; $x < count($serviceargs["commands"]); $x++) {

                $commandstring = encode_form_val($serviceargs['commands'][$x]['command']);
                $commandargs = encode_form_val($serviceargs['commands'][$x]['args']);
                $commandname = encode_form_val($serviceargs['commands'][$x]['name']);
                $is_checked = isset($services['commands'][$x])
                    ? is_checked($services['commands'][$x]) : '';

                $output .= '<tr>
        <td><input type="checkbox" class="checkbox" name="services[commands][' . $x . ']" ' . $is_checked . '></td>
        <td><input type="text" size="25" name="serviceargs[commands][' . $x . '][name]" value="' . $commandname . '" class="form-control"></td>
        <td><input type="text" size="50" name="serviceargs[commands][' . $x . '][command]" value="' . $commandstring . '" class="form-control"></td>
        <td><input type="text" size="50" name="serviceargs[commands][' . $x . '][args]"
        value="' . $commandargs . '" class="form-control"></td>
        </tr>';
            }

            $output .= '
    </table>
    <div style="height: 20px;"></div>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address");
            $hostname = grab_array_var($inargs, "hostname");
            $winrmuser = grab_array_var($inargs, "winrmuser", "");
            $winrmpassword = grab_array_var($inargs, "winrmpassword", "");
            #$osdistro = grab_array_var($inargs, "osdistro", "");

            // check for errors
            $errors = 0;
            $errmsg = array();
            if (is_valid_host_name($hostname) == false)
                $errmsg[$errors++] = _("Invalid host name.");


            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;


        case CONFIGWIZARD_MODE_GETSTAGE3HTML:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address");
            $hostname = grab_array_var($inargs, "hostname");
            $winrmuser = grab_array_var($inargs, "winrmuser", "");
            $winrmpassword = grab_array_var($inargs, "winrmpassword", "");
            $hostname = nagiosccm_replace_user_macros($hostname);
            #$osdistro = grab_array_var($inargs, "osdistro", "");

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial");
            if ($services_serial != "")
                $services = json_decode(base64_decode($services_serial), true);
            else
                $services = grab_array_var($inargs, "services");

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial");
            if ($serviceargs_serial != "")
                $serviceargs = json_decode(base64_decode($serviceargs_serial), true);
            else
                $serviceargs = grab_array_var($inargs, "serviceargs");

            $output = '

        <input type="hidden" name="ip_address" value="' . $address . '">
        <input type="hidden" name="hostname" value="' . $hostname . '">
        <input type="hidden" name="winrmuser" value="' . $winrmuser . '">
        <input type="hidden" name="winrmpassword" value="' . $winrmpassword . '">
        <input type="hidden" name="services_serial" value="' . base64_encode(json_encode($services)) . '">
        <input type="hidden" name="serviceargs_serial" value="' . base64_encode(json_encode($serviceargs)) . '">
';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:

            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:

            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:

            $hostname = grab_array_var($inargs, "hostname", "");
            $address = grab_array_var($inargs, "ip_address", "");
            #$osdistro = grab_array_var($inargs, "osdistro", "");
            $hostaddress = $address;
            $winrmuser = grab_array_var($inargs, "winrmuser", "");
            $winrmpassword = grab_array_var($inargs, "winrmpassword", "");
            $services_serial = grab_array_var($inargs, "services_serial", "");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            $services = json_decode(base64_decode($services_serial), true);
            $serviceargs = json_decode(base64_decode($serviceargs_serial), true);

            // save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostname"] = $hostname;
            $meta_arr["ip_address"] = $address;
            #$meta_arr["osdistro"] = $osdistro;
            $meta_arr["services"] = $services;
            $meta_arr["serviceargs"] = $serviceargs;
            save_configwizard_object_meta($wizard_name, $hostname, "", $meta_arr);

            $objs = array();

            $icon = viawinrm_configwizard_get_distro_icon();

            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_windowsserver_host",
                    "host_name" => $hostname,
                    "address" => $hostaddress,
                    "icon_image" => $icon,
                    "statusmap_image" => $icon,
                    "_xiwizard" => $wizard_name,
                );
            }

            // see which services we should monitor
            foreach ($services as $svc => $svcstate) {

                //echo "PROCESSING: $svc -> $svcstate<BR>\n";

                switch ($svc) {

                    case "ping":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Ping",
                            "use" => "xiwizard_windowsserver_ping_service",
                            "_xiwizard" => $wizard_name,
                        );
                        break;


                    case "commands":

                        $enabledcmds = $svcstate;
                        foreach ($enabledcmds as $pid => $pstate) {

                            $puser = $winrmuser;
                            $ppass = $winrmpassword;
                            $pname = $serviceargs["commands"][$pid]["command"];
                            $pargs = $serviceargs["commands"][$pid]["args"];
                            $pdesc = $serviceargs["commands"][$pid]["name"];

                            $checkcommand = "check_xi_by_winrm!\"" . $puser . "\"!\"" . $ppass . "\"!basic-http!\"". $pname . "\"!\"" . $pargs . "\" ";

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => $pdesc,
                                "use" => "generic-service",
                                "check_command" => $checkcommand,
                                "_xiwizard" => $wizard_name,
                            );
                        }
                        break;

                    default:
                        break;
                }
            }

            //echo "OBJECTS:<BR>";
            //print_r($objs);
            //exit();

            // return the object definitions to the wizard
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;

            break;

        default:
            break;
    }

    return $output;
}


/**
 * @param $osdistro
 *
 * @return string
 */
function viawinrm_configwizard_get_distro_icon()
{

    $icon = "win2022.png";

    return $icon;
}
